@@ -25,7 +25,7 @@ test_that("brm_data() response", {
     reference_time = "time 1",
     missing = "col_missing"
   )
-  expect_s3_class(out, "brm_data")
+  expect_s3_class(out, "brms_mmrm_data")
   expect_true(tibble::is_tibble(out))
   expect_silent(brm_data_validate(out))
   expect_true(all(is.na(out$col_response[c(2L, 3L)])))
@@ -83,6 +83,7 @@ test_that("brm_data() response", {
   expect_equal(attr(out, "brm_missing"), "col_missing")
   expect_equal(attr(out, "brm_reference_group"), "group.1")
   expect_equal(attr(out, "brm_reference_time"), "time.1")
+  expect_false(brm_data_has_subgroup(out))
 })
 
 test_that("brm_data() response with subgroup", {
@@ -133,7 +134,7 @@ test_that("brm_data() response with subgroup", {
     reference_time = "time 1",
     missing = "col_missing"
   )
-  expect_s3_class(out, "brm_data")
+  expect_s3_class(out, "brms_mmrm_data")
   expect_true(tibble::is_tibble(out))
   expect_silent(brm_data_validate(out))
   slice_filled <- dplyr::arrange(out[is.na(out$col_response), ], col_time)
@@ -223,6 +224,7 @@ test_that("brm_data() response with subgroup", {
   expect_equal(attr(out, "brm_reference_group"), "group.1")
   expect_equal(attr(out, "brm_reference_subgroup"), "subgroup.1")
   expect_equal(attr(out, "brm_reference_time"), "time.1")
+  expect_true(brm_data_has_subgroup(out))
 })
 
 test_that("brm_data() change", {
@@ -249,7 +251,7 @@ test_that("brm_data() change", {
     covariates = character(0L),
     reference_group = "group 1"
   )
-  expect_s3_class(out, "brm_data")
+  expect_s3_class(out, "brms_mmrm_data")
   expect_true(tibble::is_tibble(out))
   expect_silent(brm_data_validate(out))
   expect_true(all(is.na(out$col_response[c(2L, 3L)])))
@@ -415,3 +417,7 @@ test_that("brm_data() deprecate level_baseline", {
     class = "brm_deprecate"
   )
 })
+
+test_that("brm_data() on bad object", {
+  expect_error(brm_data_fill("Swiss cheese"), class = "brm_error")
+})
